/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: argfileparser.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef ARGFILEPARSER_INCLUDED
#define ARGFILEPARSER_INCLUDED

#include "../shared/misc_defines.h"
#include "argfiletoken.h"

#include <QString>
#include <QStringList>
#include <QHash>

namespace vhist
{

typedef QHash<QString, QString> Environment;
typedef QList<ArgfileToken> TokenList;

/**
 * A class which parses an argfile and returns a list of argument strings.
 * Variables used in the argfile are replaced using an environment
 * hashtable.
 */
class VHIST_EXTERN ArgfileParser
{
    public:
        ArgfileParser();
        ~ArgfileParser();
    
    public:
        QStringList parseArgfile(const QString& content,
                const Environment* env = NULL);
        TokenList getHighlighterList(const QString& content);
    
    private:
        TokenList splitIntoTokens(const QString& content);
        TokenList unescapeTokens(const TokenList& tokens);
        TokenList joinTokens(const TokenList& tokens);
        TokenList removeDoubleSeparators(const TokenList& tokens);
    
    private:
        QStringList findVariablesInToken(const ArgfileToken& token);
        QStringList findVariablesBackslashed(const ArgfileToken& token);
        QStringList findVariablesDoubleQuoted(const ArgfileToken& token);
        TokenList handleVariables(const TokenList& tokens,
                const Environment* env = NULL);
        
        ArgfileToken replaceVariables(const QStringList& list,
                const Environment* env);
        QString getVariableName(const QString& var);
        
        QString unescapeDoubleQuoted(const QString& text);
        QString unescapeSingleQuoted(const QString& text);
        QString unescapeBackslashed(const QString& text);
        QString escapeEverything(const QString& text);
        
        QStringList tokensToStrings(const TokenList& tokens);
};

}

#endif // ARGFILEPARSER_INCLUDED

