/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: argfilereader.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef ARGFILEREADER_INCLUDED
#define ARGFILEREADER_INCLUDED

#include <QString>

#include "../shared/misc_defines.h"
#include "../shared/logsource.h"
#include "argfile.h"
#include "argfileparser.h"

namespace vhist
{

/**
 * A class for reading VHIST argfiles. The class parses argfiles,
 * replaces variables in the argfile and generates the result in an
 * ArgFile object in a structured way.
 */
class VHIST_EXTERN ArgfileReader : LogSource
{
    public: // construction/destruction
        ArgfileReader();
        ~ArgfileReader();
    
    public: // main functionality
        ArgFile readArgfile(const QString& content,
                const Environment* env = NULL);
    
    private:
        void saveDocAttr(const QString& option, const QStringList& args,
                int pos, ArgFile& argfile);
        
        void saveStepAttr(const QString& option, const QStringList& args,
                int pos, ArgFile& argfile);
        
        void saveFileAttr(const QString& option, const QStringList& args,
                int pos, WSFile& wsFile);
        
        void saveFileFlag(const QString& option, const QStringList& args,
                int pos, WSFile& wsFile);
        
        QStringList checkOptionSignatures(const QStringList& args);
        ArgFile parseCommandline(const QStringList& args);
};

}

#endif

