/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: logsource.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef VL_LOGSOURCE_INCLUDED
#define VL_LOGSOURCE_INCLUDED

#include <QObject>
#include "misc_defines.h"

#include <QMetaType>

namespace vhist
{

/**
 * A class which is a source for log messages. The class provides an interface
 * to register loggers, which want to be notified about log messages generated
 * by this class as well as a signal to emit log messages. Each object which
 * wants to register at a LogSource object to receive log messages must provide
 * either a slot or signal with signature void log(const QString, const
 * LogSource::LogType) or void log(const QString).
 */
class VHIST_EXTERN LogSource : public QObject
{
    public:
        /**
         * An enum defining different types of log messages. A log message
         * can describe an error, a warning or be a simple status message.
         */
        enum LogType
        {
            LOG_MESSAGE /** The log message is a simple status message. */,
            LOG_WARNING /**The log message is a warning. */,
            LOG_ERROR /** The log message is an errormessage. */
        };
		    
    Q_OBJECT
    public:
        LogSource(QObject* listener = NULL);
        virtual ~LogSource();

    public:
        void publishToLogSlot(QObject* listener);
        void publishToLogSignal(QObject* listener);

    signals:
        //void log(const QString msg, const vhist::LogSource::LogType type = LOG_MESSAGE);
        void log(const QString msg, const int type = LOG_MESSAGE);
};

}

Q_DECLARE_METATYPE(vhist::LogSource::LogType)

#endif // VL_LOGSOURCE_INCLUDED

