
#ifndef MAYBE_TPL_INCLUDED
#define MAYBE_TPL_INCLUDED

/**
 * Template class which may either hold a value of type T or nothing.
 * T must have a default constructor and be copyable. 
 */
template<class T>
class Maybe
{
    public:
        /**
         * Contains the value held by the Maybe object. Use this member
         * to access the object.
         */
        T value;
        
        /**
         * Creates a new Maybe object holding a value.
         * A copy of \a value will be created.
         *
         * @param[in] value The value the object should hold.
         */
        static Maybe<T> just(const T& value)
        {
            return Maybe(value, true);
        }
        
        /**
         * Creates a new Maybe object holding nothing.
         */
        static Maybe<T> nothing()
        {
            return Maybe(T(), false);
        }
        
        /**
         * Checks if the object does not hold a value.
         *
         * @return \c true if the object does not hold a value.
         */
        bool isNothing()
        {
            return !success;
        }
        
        /** Checks if the object does hold a value.
         *
         * @return \c true if the object does hold a value.
         */
        bool hasValue()
        {
            return success;
        }
    
    private:
        Maybe(const T& value, bool success)
        {
            this->value = value;
            this->success = success;
        }
        
        bool success;
};

#endif // MAYBE_TPL_INCLUDED
