/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: summaryreader.h 2792 2013-05-06 11:25:27Z michael $
 */

#ifndef SUMMARYREADER_INCLUDED
#define SUMMARYREADER_INCLUDED

#include "../shared/misc_defines.h"
#include "../shared/tripple.tpl"

#include <QObject>
#include <QSharedPointer>
#include <QString>

#include "tags.h"
#include "summary.h"

namespace vhist
{

typedef Tripple<QString, QString, QSharedPointer<QString> > Attribute;

/**
 * A class reading and evaluating summary files. A summary file is
 * an xml file containing all information about one workflowstep.
 * A tool creating VHIST files will generate one summary file for every
 * section it creates and embed this file as the last embedded file in
 * the section.
 */
class VHIST_EXTERN SummaryReader: public QObject
{
    Q_OBJECT
    public: // construction, destruction and assignment
        SummaryReader();
        ~SummaryReader();
    
    public:
        Summary readSummary(const QString& data);
    
    private:
        Summary parseSummary(const QString& content);
        Workflowstep parseWorkflowstep(const QString& content);
        FileSummary parseFile(const QString& content);
        Tripple<UserAttributes,UserAttributeOrder,UserAttributeUnits>
                parseUserAttributes(const QString& content);
        Attribute parseAttribute(const QString& content);
};

Summary VHIST_EXTERN makeSummaryFromTags(const SectionTag& tag, const FileTagList& files);
FileSummary VHIST_EXTERN makeFileSummaryFromFileTag(const FileTag& tag);

void VHIST_EXTERN printFileSummary(const FileSummary& f);
void VHIST_EXTERN printSummary(const Summary& s);

QString VHIST_EXTERN makeFileHash(const FileSummary& f, int sectionID);

} // namespace

#endif

