/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistrunner.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef VL_VHISTRUNNER_INCLUDED
#define VL_VHISTRUNNER_INCLUDED


#include "../shared/misc_defines.h"
#include "../shared/logsource.h"

#include <QString>
#include <QStringList>


namespace vhist
{

/**
 * Executes an VHIST argfile using \e vhistadd. The class searches for
 * a \e vhistadd executable in the given path list. Both, the native
 * python version and the \e py2exe version of \e vhistadd are supported.
 */
class VHIST_EXTERN VhistRunner : public LogSource
{
	public:
    Q_OBJECT
    
    public:
        /**
         * Errorcodes generated by VhistaddRunner.
         */
        enum VHISTADD_ERROR
        {
            /// no vhistadd binary was found in the search path.
            VHISTADD_NOT_FOUND = -1000,
            /// vhistadd could not be started successfully.
            VHISTADD_FAILED_TO_START = -1001
        };
    
    public:
        VhistRunner(const QString &vhistrunstump, QObject* listener = NULL);
        virtual ~VhistRunner();
    
    public:
        void setSearchPaths(const QStringList& paths);
        void setUsePy2Exe(bool state);
    
    public:
        QString findPython();
        QString findVhistrun();

		bool getUsePy2Exe() const
			{return m_usePy2Exe; }
		void clearOutputRest()
			{m_outputRest.clear(); }
        void processOutputLine(const QString& line);
        void processOutput(const QString& output, bool finish = false);
        
        virtual int runProcess(const QString& argfile) = 0;
    
    signals:
        void intercept();
    
    private:
        QString searchInPath(const QString& path, const QString& file);
    
    private:
		QString m_vhistrun;
		QString m_vhistrunstump;
        QString m_outputRest;
    
    private:
        bool m_usePy2Exe;
        QStringList m_searchPaths;
};

}

#endif // VL_VHISTRUNNER_INCLUDED

