/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: pathexpander.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef PATHEXPANDER_H
#define PATHEXPANDER_H

#include <QLinkedList>

#include "workflowtree.h"
#include "path.h"
#include <QHash>

namespace vhist
{

class CostMatrix;

class VHIST_EXTERN PathExpander
{
    private:
        const WorkflowTree* m_tree1;
        const WorkflowTree* m_tree2;
        const CostMatrix* m_costs;

    public: // construction & destruction
        PathExpander(const WorkflowTree* tree1, const WorkflowTree* tree2,
                const CostMatrix* costs);
        ~PathExpander();

    public:
        QLinkedList<NodeId> nodeIds(TreeId treeNo) const;

    public: // getter
        bool parentChanged(NodeId node1, const Associations& associations,
                const Associations& revAssociations) const;
        float costForAssociation(NodeId node1, NodeId node2,
                const Associations& associations,
                const Associations& revAssociations) const;
        float costForUnmatchedNodes(int numOfUnmatchedNodes) const;
        float heuristicCosts(const QLinkedList<NodeId>& nodes1,
                const QLinkedList<NodeId>& nodes2) const;
        bool calcParentCostsLater(NodeId node1, NodeId node2,
                const Associations& associations,
                const Associations& revAssociations) const;
        float calcMissingParentCosts(const Associations& associations,
                const Associations& revAssociations,
                const QHash<NodeId, bool>& parentCostsMissing) const;
};

} // namespace vhist

#endif // PATHEXPANDER_H
