/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistdiffresult.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef VHISTDIFFRESULT_H
#define VHISTDIFFRESULT_H

#include <QHash>
#include <QList>

#include "../shared/misc_defines.h"
#include "diffdefines.h"
#include "path.h"

namespace vhist
{

class Path;


class VHIST_EXTERN VhistDiffResult
{
    public:
        VhistDiffResult();
        ~VhistDiffResult();
        VhistDiffResult(const VhistDiffResult& rhs);
        VhistDiffResult(const Associations& associations,
                const Associations& revAssociations,
                const QHash<NodeId, bool> nodeMovedTree1,
                const QHash<NodeId, bool> nodeMovedTree2);

        VhistDiffResult& operator=(const VhistDiffResult& rhs);

    public: // getter
        NodeId association(NodeId nodeId, TreeId treeId) const;
        bool associated(NodeId nodeId, TreeId treeId) const;
        bool parentChanged(NodeId nodeId, TreeId treeId) const;

    private:
        Associations m_associations;
        Associations m_revAssociations;
        QHash<NodeId, bool> m_nodeMovedTree1;
        QHash<NodeId, bool> m_nodeMovedTree2;
};

typedef QList<VhistDiffResult> VhistDiffResults;

}

#endif // VHISTDIFFRESULT_H
