/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistfilemanager.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef VHISTFILEMANAGER_INCLUDED
#define VHISTFILEMANAGER_INCLUDED

#include <QString>
#include <QSharedPointer>
#include <QHash>

#include "../shared/misc_defines.h"
#include "../shared/logsource.h"
#include "tempvhistfile.h"

namespace vhist
{

class VHIST_EXTERN VhistFileManager : public LogSource
{
    Q_OBJECT
    public:
        VhistFileManager(QObject* listener = NULL);
        VhistFileManager(const VhistFileManager& rhs);
        ~VhistFileManager();

        VhistFileManager& operator=(const VhistFileManager& rhs);

    public:
        SPVhistParser addVhistFile(const QString& filename, bool deleteFile);
        SPVhistParser addVhistFile(SPVhistParser vhistfile,
                const QString& filename);
        SPVhistParser parser(const QString& filename);
        void clear();

    private:
        QHash<QString, QSharedPointer<TempVhistfile> > m_files;
};


} // namespace vhist

#endif // VHISTFILEMANAGER_INCLUDED
