/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: workflowtree.h 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#ifndef WORKFLOWTREE_INCLUDED
#define WORKFLOWTREE_INCLUDED

#include "../shared/misc_defines.h"
#include "../unvhistlib/vhistparser.h"
#include "../shared/logsource.h"
#include "diffdefines.h"
#include "workflowtreenode.h"
#include "vhistfilemanager.h"

namespace vhist
{

class VHIST_EXTERN WorkflowTree : public LogSource
{
    Q_OBJECT

    private:
        typedef QHash<NodeId, WorkflowTreeNode> TreeNodeHash;
        typedef QHash<NodeId, QList<NodeId> > ChildHash;

    public: // construction/destruction
        WorkflowTree(QObject* listener = NULL);
        WorkflowTree(const WorkflowTree& rhs);
        ~WorkflowTree();

        WorkflowTree& operator=(const WorkflowTree& rhs);

    public: // structural modifications of tree
        NodeId addTreeNode(NodeId parentId, const Workflowstep& wfs,
                const QString& filename, const QString& tmpfile);
        void clear();

    public: // creation of tree from VHIST files
        void treeFromVhistFile(SPVhistParser vhistFile,
                 const QString& filename = "rootfile.vhist");
        void treeFromVhistFile(const QString& vhistFile);

    public: // getter
        const Workflowstep* workflowStep(NodeId nodeId) const;
        int parent(NodeId nodeId) const;
        QString vhistfile(NodeId nodeId) const;
        QString fileOnDisk(NodeId nodeId) const;
        QList<NodeId> children(int nodeId) const;
        SPVhistParser vhistParser(NodeId nodeId);

        QList<NodeId> serializedTree() const;
        NodeId root() const;
        QList<NodeId> nodeIds() const;
        QLinkedList<NodeId> nodeIdsLinked() const;


    private: // tree filling helper
        QString extractFile(SPVhistParser vhistFile, int tagId);
        void fillTree(SPVhistParser vhistFile, NodeId parent,
                const QString& vhistFilename,
                const QString& fileOnDisk);

    private: // structural helper
        void addTreeNode(const WorkflowTreeNode& node);
        QList<NodeId> serializeFromRoot(const NodeId& root) const;

    private: // member variables
        NodeId m_nextId;
        TreeNodeHash m_nodes;
        ChildHash m_children;
        VhistFileManager m_filemanager;
};

} // namespace vhist

#endif // WORKFLOWTREE_H
