/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: evaloperators.h 2168 2011-05-30 09:09:54Z ahuesgen $
 */

#ifndef EVALOPERATORS_H
#define EVALOPERATORS_H

#include "returnvalue.h"
#include "../shared/misc_defines.h"

namespace vhist
{

// equality operators
ReturnValue evaluateEq(const ReturnValue& v0, const ReturnValue& v1);// {return ReturnValue(RETURN_VALUE_BOOL, "");}
ReturnValue evaluateNEq(const ReturnValue& v0, const ReturnValue& v1);

// logical operators
ReturnValue evaluateNot(const ReturnValue& v0);
ReturnValue evaluateAnd(const ReturnValue& v0, const ReturnValue& v1);
ReturnValue evaluateOr(const ReturnValue& v0, const ReturnValue& v1);

// numeric operators
ReturnValue evaluateLess(const ReturnValue& v0, const ReturnValue& v1);
ReturnValue evaluateLessEq(const ReturnValue& v0, const ReturnValue& v1);
ReturnValue evaluateGreater(const ReturnValue& v0, const ReturnValue& v1);
ReturnValue evaluateGreaterEq(const ReturnValue& v0, const ReturnValue& v1);
ReturnValue evaluateIsApprox(const ReturnValue& v0, const ReturnValue& v1);

// string matching operators
ReturnValue evaluateContains(const ReturnValue& v0, const ReturnValue& v1);
ReturnValue evaluateMatches(const ReturnValue& v0, const ReturnValue& v1);

}

#endif // EVALOPERATORS_H
