/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: expressionvalue.h 2168 2011-05-30 09:09:54Z ahuesgen $
 */

#ifndef EXPRESSIONVALUE_H
#define EXPRESSIONVALUE_H

#include "../shared/misc_defines.h"

#include <QString>
#include <QtDebug>

namespace vhist
{

struct Variable
{
    QString name;
    QString attribute;
};


struct Operator
{
    QString operatorValue;
    int precedence;
    char association; // 'l' for left, 'r' for right
    int numArguments;
};


enum ValueType
{
    VALUE_TYPE_VARIABLE = 1,
    VALUE_TYPE_STRING = 2,
    VALUE_TYPE_OPERATOR = 4
};


class ExpressionValue
{
public:
    ExpressionValue();
    
public:
    bool isVariable() const;
    bool isString() const;
    bool isOperator() const;

public:
    QString originalText() const;
    int position() const;
    int length() const;

    QString string() const;

    QString operatorName() const;
    int precedence() const;
    void setPrecedence(int newPrecedence);
    int numArgs() const;
    char association() const;

    QString variableName() const;
    QString variableAttribute() const;

public:
    void setVariableName(const QString& name);

    

private:
    ValueType m_type;
    Variable m_var;
    QString m_string;
    Operator m_operator;
    QString m_originalText;
    int m_pos;
    
public:
    static ExpressionValue fromString(const QString& string, int pos = -1,
            const QString& originalText = "");
    static ExpressionValue fromVariable(const QString& variableName,
            const QString attribute, int pos = -1,
            const QString& originalText = "");
    static ExpressionValue fromVariable(const QString& variable,
            int pos = -1, const QString& originalText = "");
    static ExpressionValue fromOperator(const QString& string,
            int pos = -1, const QString& originalText = "");
};

}

QDebug operator<<(QDebug dbg, const vhist::ExpressionValue &v);

#endif // EXPRESSIONVALUE_H
