/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistfindquery.h 2168 2011-05-30 09:09:54Z ahuesgen $
 */

#ifndef FINDQUERY_H
#define FINDQUERY_H

#include "../shared/misc_defines.h"
#include "../unvhistlib/summary.h"
#include "../unvhistlib/vhistparser.h"

#include "queryerror.h"
#include "cnf.h"

#include <QString>
#include <QHash>

namespace vhist
{

/**
 * This class represents a query concerning properties of VHIST files.
 * After creating an instance of this class, you can then check whether
 * or not a VHIST file matches the requirements specified in the query.
 */
class VHIST_EXTERN VhistFindQuery
{
public:
    VhistFindQuery(const QString& query = "");
    ~VhistFindQuery();

public:
    bool setQuery(const QString& query);
    bool matches(VhistParser& parser, QHash<QString, QString>*
            associations) const;
    bool matches(const Summaries& summaries, QHash<QString, QString>*
            associations) const;

public:
    bool isValid() const;
    QueryErrors getErrors() const;

private:
    QString m_query;
    QueryErrors m_errors;
    ConjunctiveNormalForm m_cnf;
};

}

#endif // FINDQUERY_H
