/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: utils.cpp 2794 2013-05-06 11:26:53Z michael $
 */

#include "../../include/shared/utils.h"

#include <QCoreApplication>
#include <QDir>



/**
 * The directory in which the application binary is located. On Mac OS X,
 * the MacOS/ directory within the bundle is returned. On all other
 * platforms, the bin directory is returned.
 * 
 * @return The path to the directory in which the binary is located.
 */
QString vhist::getBinaryDir()
{
    QString path(QCoreApplication::applicationDirPath());
    QDir dirPath(path);
    return dirPath.canonicalPath();
}


/**
 * The directory of the application. On Mac OS X, this is the directory
 * which contains the bundle, on all other platforms, \c getApplicationDir()
 * is the same as \c getBinaryDir().
 * 
 * @return The path to the application directory.
 */
QString getApplicationDir()
{
    QString dirStr = vhist::getBinaryDir();
    
#ifdef Q_OS_MAC    
    // remove <appname>.app/Contents/MacOS from path
    if (dirStr.endsWith(QString("/Contents/MacOS")))
    {
        QDir appDir(dirStr);
        appDir.cdUp();
        appDir.cdUp();
        appDir.cdUp();
        // path is already canonical
        dirStr = appDir.absolutePath();
    }
#endif // end of Mac specific code
    
    return dirStr;
}

