// $Id: xmlit.cpp 2739 2013-04-24 08:54:08Z michael $
//#include "../include/no_ascii_cast.h"
#ifdef USING_PCH
//#include "stable.h"
#endif

#include "../../include/unvhistlib/xmlit.h"

#ifndef USING_PCH
#include <QObject>
#include <QTextStream>
#endif

using namespace vhist;

//    ***********************************************************************
//! useful for iterating through a chunk of XML code, level by level
/*
     QString qs = "<root>"
        "<abc>tralal</abc>"
        "<def>adsfsadfasdf</def>"
        "</root>";

    VXMLLevel1Iter xi;
    for (xi=qs; xi; ++xi) {

        if (xi.isTag("adc"))
            do(xi.getData());
        else if (xi.isTag("def"))
            do(xi.getData());

        //QString tag = xi.getTag();     // "root"
        //QString data = xi.getData();   // "<abc>...</def>"
        //...
    }
*/

VXMLLevel1Iter::VXMLLevel1Iter ( const QString &szXML ):
m_bWithMsg(true)
    {Init(szXML); }

VXMLLevel1Iter::VXMLLevel1Iter ( ):
m_bWithMsg(true)
    {Init(QString()); }

VXMLLevel1Iter::VXMLLevel1Iter ( const VXMLLevel1Iter &anOther)
    {Copy(anOther);}


VXMLLevel1Iter::~VXMLLevel1Iter ( )
    {}


VXMLLevel1Iter &VXMLLevel1Iter::operator=(const VXMLLevel1Iter &anOther)
{
    Copy(anOther);
    return *this;
}

//! Re-initializes the iterator and starts the iteration
VXMLLevel1Iter &VXMLLevel1Iter::operator=(const QString &szReInit)
{
    int iSize;
    QChar *dummy;
    Init(szReInit);
    getNextDataBlock(&dummy,&iSize);
    return *this;
}


void VXMLLevel1Iter::Init (const QString &pszXML)
{
    m_bOK=true;
    m_szXMLOrig=pszXML;
    m_szXML=QString::fromStdWString(m_szXMLOrig.toStdWString()); //unicode safe deep copy
    m_pcXML=m_szXML.data();  
    m_iLength=m_szXML.count();
    m_iCurPos=0;
    pTag=m_pcXML;
    pData=m_pcXML;
    pEnd=m_pcXML;
    m_iDataBlockStart=-1;
    m_iDataBlockEnd=-1;
    m_iTagSize=m_szXML.count();
    m_iDataSize=m_szXML.count();
}

void VXMLLevel1Iter::Copy ( const VXMLLevel1Iter &anOther)
{
    m_bOK=anOther.m_bOK;
    m_bWithMsg=anOther.m_bWithMsg;
    m_szXMLOrig=anOther.m_szXMLOrig;
    m_szXML=QString::fromStdWString(m_szXMLOrig.toStdWString()); //unicode safe deep copy
    m_pcXML=m_szXML.data();  
    m_iLength=m_szXML.count();
    m_iCurPos=anOther.m_iCurPos;
    pTag=anOther.pTag;
    pData=anOther.pData;
    pEnd=anOther.pEnd;
    m_iDataBlockStart=anOther.m_iDataBlockStart;
    m_iDataBlockEnd=anOther.m_iDataBlockEnd;
    m_iTagSize=anOther.m_iTagSize;
    m_iDataSize=anOther.m_iDataSize;
}

const QString VXMLLevel1Iter::operator++(void)
{
    QChar *dummy;
    int iDataSize;
    getNextDataBlock(&dummy,&iDataSize);
    return m_bOK ? QString::fromRawData(pData,iDataSize) : QString();
}

const QString VXMLLevel1Iter::operator++(int)
{
    bool bOk=m_bOK;
    QChar *pSave=pData,*dummy;
    int iSaveSize=m_iDataSize,iDataSize;
    getNextDataBlock(&dummy,&iDataSize);
    return bOk ? QString::fromRawData(pSave,iSaveSize) : QString();
}

//! returns pointer to zero-terminated tag name, szData points to data segment without tag braces.
void VXMLLevel1Iter::getNextDataBlock ( 
    QChar **pqcData, int *iSizeData)
{
    int        iTagStart, iTagEnd, iDataEnd, iPos;
    bool    bSpecialCase = false;

    m_iDataBlockStart = 0;
    m_iDataBlockEnd = 0;

    m_iTagSize=0;
    m_iDataSize=0;

    do
    {
        bSpecialCase = false;

        if ( !m_bOK ) {
            return; // NULL;
        }
        
        iTagStart = m_szXML.indexOf ( QLatin1String("<"), m_iCurPos );
        if ( iTagStart < 0 ) {
            m_bOK=false;
            return; // NULL;
        }
        m_iCurPos = iTagStart + 1;
        
        iTagEnd = m_szXML.indexOf ( QLatin1String(">"), m_iCurPos ); 
        
        if ( iTagEnd < 0 )
        {
            if (m_bWithMsg)
                //lprintf(VError_Msg::errorMsg(QLatin1String("malformed XML, missing >.\n")));
            m_bOK=false;
            return; // NULL;
        }
        
        // check for special cases, and just ignore the code between brackets
        
        if ( m_pcXML[ m_iCurPos ] == QLatin1Char('?') )    // xml version, e.g. <?xml version="1.0" standalone="yes" ?>
        {
            m_bOK = true;
            bSpecialCase = true;
            iPos = m_szXML.indexOf ( QLatin1String("?>"), m_iCurPos + 1 );
            if ( iPos <= 0 )
            {
                if (m_bWithMsg)
                    //lprintf(VError_Msg::errorMsg(QLatin1String("malformed XML, tag \"<?\" is not closed.\n")));
                m_bOK=false;
                return; // NULL;
            }
            m_iCurPos = iPos + 1;
        }
        else if ( m_pcXML[ m_iCurPos ] == QLatin1Char('!') )    // xml comment, e.g. <!-- comment -->
        {
            m_bOK = true;
            bSpecialCase = true;
            iPos = m_szXML.indexOf ( QLatin1String("-->"), m_iCurPos + 1 );
            if ( iPos <= 0 )
            {
                if (m_bWithMsg)
                    //lprintf(VError_Msg::errorMsg(QLatin1String("malformed XML, comment tag \"<!\" is not closed.\n")));
                m_bOK=false;
                return; // NULL;
            }
            m_iCurPos = iPos + 1;
        }

    } while ( bSpecialCase );


    m_iTagSize=iTagEnd-iTagStart-1;
    m_iCurPos = iTagEnd + 1;
    pTag = &m_pcXML[ iTagStart + 1 ];
    
    if ( pTag [ 0 ] == QLatin1Char('/') )
    {
        if (m_bWithMsg)
            //lprintf(VError_Msg::errorMsg(QObject::tr("malformed XML, end tag \"%1\" only.\n").arg(QString::fromRawData(pTag,m_iTagSize))));
        m_bOK=false;
        return; // NULL;
    }
    
    m_pcXML[ iTagEnd ] = QChar::Null;
    if ( m_pcXML[ iTagEnd - 1 ] == QLatin1Char('/') )
    {
        // this is XML short for tag without data, ie. "flag"
        m_iTagSize--;
        m_pcXML[ iTagEnd - 1 ] = QChar::Null;
        *pqcData = &m_pcXML[ iTagEnd - 1 ]; // points to '0'
        m_iCurPos = iTagEnd + 1;

        m_bOK=true;
        pEnd=&m_pcXML[m_iCurPos];
        m_iDataBlockStart = iTagEnd + 1;
        m_iDataBlockEnd = m_iDataBlockStart;
        m_iDataSize=m_iDataBlockEnd-m_iDataBlockStart;
        *iSizeData=m_iDataSize;
        pData=*pqcData; // !sv *?
        return; // pTag;
    }

    // if we get here, there should be a regular closing tag
    // of form </tag>

    *pqcData = &m_pcXML[ iTagEnd + 1 ];
    m_iDataBlockStart = iTagEnd + 1;
    QString szTT=QString::fromRawData(pTag,m_iTagSize);
    QString szEndTag = QString::fromLatin1("</%1>").arg(szTT);

    int inside=1;
    int iStart,iEnd;
    bool completely_outside=false;
    QString szRecursivStartTag = QString::fromLatin1("<%1>").arg(szTT);

    iDataEnd=m_iCurPos;
    // jump over recursive tags
    while (!completely_outside && (inside > 0)) {
        iStart=m_szXML.indexOf ( szRecursivStartTag, m_iCurPos );
        iEnd=m_szXML.indexOf ( szEndTag, m_iCurPos );
        completely_outside=(iStart < 0) && (iEnd < 0);
        if (!completely_outside) {
            if (iStart < 0) {
                inside--;
                iDataEnd=iEnd;
                m_iCurPos=iDataEnd+szEndTag.size();
            }
            else if (iEnd < 0) {
                inside++;
                m_iCurPos=iStart+szRecursivStartTag.size();
            }
            else if (iStart > iEnd)    {
                inside--;
                iDataEnd=iEnd;
                m_iCurPos=iDataEnd+szEndTag.size();
            }
            else {
                inside++;
                m_iCurPos=iStart+szRecursivStartTag.size();
            }
        }
    }

    if ( completely_outside )
    {
        if (m_bWithMsg)
            //lprintf(VError_Msg::errorMsg(QObject::tr("malformed XML, end tag \"%1\" missing.\n").arg(szEndTag)));
        m_bOK=false;
        return; // NULL;
    }
    
    m_iDataBlockEnd = iDataEnd;
    m_iDataSize=m_iDataBlockEnd-m_iDataBlockStart;
    m_pcXML[ iDataEnd ] = QChar::Null;

    m_bOK=true;
    pEnd=&m_pcXML[m_iCurPos];

    *iSizeData=m_iDataSize;
    pData=*pqcData; //!sv *?
    return; // pTag;
}

QString VXMLLevel1Iter::replaceCurrentTag(QString szReplacement)
{
    QString szOrig=m_szXMLOrig;
    return szOrig.replace(m_iDataBlockStart,
        m_iDataBlockEnd-m_iDataBlockStart,szReplacement);
}

const QString VXMLLevel1Iter::getTagAndData(void) const
    {return QString::fromLatin1("<%1>%2</%1>").arg(getTag()).arg(getData()); }

bool VXMLLevel1Iter::dataTo3Int(int &i1, int &i2, int &i3) const 
{
    QString szStreamConvert = getData();
    QTextStream in (&szStreamConvert);
    in >> i1 >> i2 >> i3;
    return (in.status() == QTextStream::Ok);
}

bool VXMLLevel1Iter::dataTo3Double(double &r1, double &r2, double &r3) const 
{
    QString szStreamConvert = getData();
    QTextStream in (&szStreamConvert);
    in >> r1 >> r2 >> r3;
    return (in.status() == QTextStream::Ok);
}

