/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: costmatrix.cpp 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#include "../../include/vhistdifflib/costmatrix.h"

#include "../../include/vhistdifflib/workflowtree.h"
#include "../../include/vhistdifflib/workflowstepcosts.h"

#include <QtDebug>

using namespace vhist;

const Costs CostMatrix::DEFAULT_COSTS_WRONG_PARENT = 10;
const Costs CostMatrix::DEFAULT_COSTS_UNMATCHED = 15;

CostMatrix::CostMatrix(const WorkflowTree& tree1, const WorkflowTree& tree2,
        const WorkflowstepCosts* wfsCosts)
        : m_costsUnmatched(DEFAULT_COSTS_UNMATCHED),
          m_costsWrongParent(DEFAULT_COSTS_WRONG_PARENT)
{
    QList<NodeId> nodes1(tree1.nodeIds());
    QList<NodeId> nodes2(tree2.nodeIds());

    for (int i = 0; i < nodes1.size(); ++i)
        for (int j = 0; j < nodes2.size(); ++j)
        {
            NodeId id1 = nodes1[i];
            NodeId id2 = nodes2[j];

            float costs = wfsCosts->costs(*tree1.workflowStep(id1),
                    *tree2.workflowStep(id2));
            m_costs.insert(QPair<NodeId, NodeId>(id1, id2), costs);
        }
}


CostMatrix::~CostMatrix()
{
}


Costs CostMatrix::costs(NodeId id1, NodeId id2) const
{
    return m_costs.value(QPair<NodeId, NodeId>(id1, id2), m_costsUnmatched);
}


Costs CostMatrix::costsWrongParent() const
{
    return m_costsWrongParent;
}


Costs CostMatrix::costsUnmatchedNode() const
{
    return m_costsUnmatched;
}

void CostMatrix::printMe()
{
    for (QHash<QPair<NodeId, NodeId>, Costs>::Iterator it = m_costs.begin();
            it != m_costs.end(); it++)
    {
        qDebug() << it.key() << it.value();
    }
}
