/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistdiffresult.cpp 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#include "../../include/vhistdifflib/vhistdiffresult.h"

#include "../../include/vhistdifflib/path.h"

using namespace vhist;


VhistDiffResult::VhistDiffResult()
{
}


VhistDiffResult::~VhistDiffResult()
{
}


VhistDiffResult::VhistDiffResult(const VhistDiffResult& rhs)
{
    *this = rhs;
}


VhistDiffResult::VhistDiffResult(const Associations& associations,
        const Associations& revAssociations,
        const QHash<NodeId, bool> nodeMovedTree1,
        const QHash<NodeId, bool> nodeMovedTree2)
{
    m_associations = associations;
    m_revAssociations = revAssociations;
    m_nodeMovedTree1 = nodeMovedTree1;
    m_nodeMovedTree2 = nodeMovedTree2;
}


VhistDiffResult& VhistDiffResult::operator=(const VhistDiffResult& rhs)
{
    if (this != &rhs)
    {
        m_associations = rhs.m_associations;
        m_revAssociations = rhs.m_revAssociations;
        m_nodeMovedTree1 = rhs.m_nodeMovedTree1;
        m_nodeMovedTree2 = rhs.m_nodeMovedTree2;
    }
    return *this;
}


NodeId VhistDiffResult::association(NodeId nodeId, TreeId treeId) const
{
    if (treeId == FIRST_TREE)
        return m_associations.value(nodeId, INVALID_NODE);
    else if (treeId == SECOND_TREE)
        return m_revAssociations.value(nodeId, INVALID_NODE);
    else
        return INVALID_NODE;
}


bool VhistDiffResult::associated(NodeId nodeId, TreeId treeId) const
{
    if (treeId == FIRST_TREE)
        return m_associations.value(nodeId, NULL_NODE) == NULL_NODE;
    else if (treeId == SECOND_TREE)
        return m_revAssociations.value(nodeId, NULL_NODE) == NULL_NODE;
    else
        return false;
}


bool VhistDiffResult::parentChanged(NodeId nodeId, TreeId treeId) const
{
    if (treeId == FIRST_TREE)
        return m_nodeMovedTree1.value(nodeId, false);
    else if (treeId == SECOND_TREE)
        return m_nodeMovedTree2.value(nodeId, false);
    else
        return false;
}


