/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistfilemanager.cpp 2179 2011-06-24 09:03:24Z ahuesgen $
 */

#include "../../include/vhistdifflib/vhistfilemanager.h"

#include "../../include/unvhistlib/vhistparser.h"

using namespace vhist;


VhistFileManager::VhistFileManager(QObject* listener)
    : LogSource(listener)
{
}


VhistFileManager::~VhistFileManager()
{
}


VhistFileManager::VhistFileManager(const VhistFileManager& rhs)
    : LogSource()
{
    *this = rhs;
}


VhistFileManager& VhistFileManager::operator=(const VhistFileManager& rhs)
{
    if (this != &rhs)
        m_files = rhs.m_files;
    return *this;
}


SPVhistParser VhistFileManager::addVhistFile(SPVhistParser vhistfile,
        const QString& filename)
{
    if (!m_files.contains(filename))
    {
        QSharedPointer<TempVhistfile> newFile(new TempVhistfile());
        newFile->m_deleteFile = false;
        newFile->m_fileOnDisk = filename;
        newFile->m_vhistfile = vhistfile;
        vhistfile->publishToLogSignal(this);
        m_files.insert(filename, newFile);
    }
    return m_files[filename]->m_vhistfile;
}


SPVhistParser VhistFileManager::addVhistFile(const QString& filename,
        bool deleteFile)
{
    if (!m_files.contains(filename))
    {
        SPVhistParser parser(new vhist::VhistParser(vhist::MODE_ROBUST, NULL));
        parser->publishToLogSignal(this);
        if (parser->open(filename))
        {
            QSharedPointer<TempVhistfile> newFile(new TempVhistfile());
            newFile->m_deleteFile = deleteFile;
            newFile->m_fileOnDisk = filename;
            newFile->m_vhistfile = parser;
            m_files.insert(filename, newFile);

            return parser;
        }
    }
    return SPVhistParser(NULL);
}


SPVhistParser VhistFileManager::parser(const QString& filename)
{
    if (m_files.contains(filename))
        return m_files[filename]->m_vhistfile;
    return SPVhistParser(NULL);
}


void VhistFileManager::clear()
{
    m_files.clear();
}
