/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: expressiontreenode.cpp 2169 2011-05-30 09:11:44Z ahuesgen $
 */

#include "../../include/vhistfindlib/expressiontreenode.h"

using namespace vhist;

ExpressionTreeNode::ExpressionTreeNode() : m_pos(0)
{
}


SPExpressionTreeNode ExpressionTreeNode::cloneTree()
{
    SPExpressionTreeNode copy(new ExpressionTreeNode());
    if (copy)
    {
        copy->m_value = m_value;
        copy->m_originalText = m_originalText;
        copy->m_pos = m_pos;
        copy->m_variables = m_variables;
        for (int i = 0; i < m_children.length(); ++i)
        {
            SPExpressionTreeNode child = m_children.at(i)->cloneTree();
            if(child)
                copy->m_children.append(child);
            else
                return SPExpressionTreeNode(NULL);
        }
    }
    return copy;
}


void printTree(QDebug dbg, SPExpressionTreeNode node, int indent)
{
    if (node != NULL)
    {
        QString indentStr = QString("    ").repeated(indent);
        dbg << indentStr << node->m_value << "\n";
        for (int i = 0; i < node->m_children.length(); ++i)
            printTree(dbg, node->m_children.at(i), indent + 1);
    }
}


QDebug operator<<(QDebug dbg, SPExpressionTreeNode v)
{
    printTree(dbg, v, 0);
    return dbg;
}
