/* 
This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

 $Id: vhistfindquery.cpp 2172 2011-06-06 07:27:14Z ahuesgen $
 */

#include "../../include/vhistfindlib/vhistfindquery.h"

#include "../../include/vhistfindlib/parsequery.h"
#include "../../include/vhistfindlib/constructcnf.h"
#include "../../include/vhistfindlib/evaluatecnf.h"

#include "../../include/unvhistlib/summarycreator.h"


using namespace vhist;

VhistFindQuery::VhistFindQuery(const QString& query) : m_query(query)
{
    setQuery(query);
}


VhistFindQuery::~VhistFindQuery()
{
}


bool VhistFindQuery::setQuery(const QString& query)
{
    m_cnf.clear();
    SPExpressionTreeNode tree = parseQuery(query, m_errors);
    if (tree != NULL)
        m_cnf = constructCNF(tree);
    return tree != NULL;
}


bool VhistFindQuery::matches(VhistParser& parser,
        QHash<QString, QString>* associations) const
{
    SummaryCreator creator;
    Summaries summaries = creator.createSummaries(parser);
    return matches(summaries, associations);
}


bool VhistFindQuery::matches(const Summaries& summaries,
        QHash<QString, QString>* associations) const
{
    QHash<QString, QString> tmp;
    if (associations == NULL)
        associations = &tmp;

    return evaluateCNF(summaries, m_cnf, *associations);
}


bool VhistFindQuery::isValid() const
{
    return !containsErrors(m_errors);
}


QueryErrors VhistFindQuery::getErrors() const
{
    return m_errors;
}
